- test:
    timeout:
      minutes: {{ timeout }}
    definitions:
    - repository: {{ test_definitions }}
      lava-signal: kmsg
      from: url
      compression: zstd
      path: automated/linux/v4l2/v4l2-compliance.yaml
      parameters:
        VIDEO_DRIVER: vivid.ko
      name: {{ name }}

