- test:
    timeout:
      minutes: 30
    interactive:
    - name: "boot-debian-dt"
      prompts: ["login:", "Password:", "root@morello:~#"]
      script:
      - command: "\x1b[A\r\n"
      - command: "root"
      - command: "morello"
        name: boot-debian-dt
